"""`libscarab` Python wrapper"""

import json
from ctypes import c_int, c_ulong

from .loader import Library
from .types import make_c_mpz_t, clear_c_mpz_t, assign_c_mpz_t, compare_c_mpz_t, \
                   make_c_fhe_pk_t, clear_c_fhe_pk_t, \
                   make_c_fhe_sk_t, clear_c_fhe_sk_t, \
                   serialize_c_fhe_pk_t, deserialize_c_fhe_pk_t, \
                   serialize_c_mpz_t, deserialize_c_mpz_t


lib_scarab = Library.load('scarab')


class EncryptedBit(object):

    """Encrypted bit"""

    def __init__(self, pk, value):
        """Create encrypted bit

        :param pk: :class:`~PublicKey` object
        :param value: initialized mpz_t object or serialized mpz_t
        """
        self._pk = pk
        if isinstance(value, str):
            self._as_parameter_ = deserialize_c_mpz_t(value)
        else:
            self._as_parameter_ = value

    def __xor__(self, other):
        """Homomorphic XOR"""
        c = make_c_mpz_t()
        lib_scarab.fhe_add(c, self, other, self._pk)
        return EncryptedBit(self._pk, c)

    def __and__(self, other):
        """Homomorphic AND"""
        c = make_c_mpz_t()
        lib_scarab.fhe_mul(c, self, other, self._pk)
        return EncryptedBit(self._pk, c)

    def __del__(self):
        """Clear the mpz_t value"""
        clear_c_mpz_t(self)

    def __add__(self, other):
        """XOR alias"""
        return self.__xor__(other)

    def __str__(self):
        """Serialize encrypted bit to string"""
        return serialize_c_mpz_t(self._as_parameter_)

    def recrypt(self, sk):
        """Recrypt a cyphertext (refreshing it)"""
        lib_scarab.fhe_recrypt(self, self._pk, sk)


class EncryptedArray(object):

    """Encrypted array (ciphertext)"""

    def __init__(self, n, pk, array=None):
        """Construct empty array

        :param n:     array size
        :param pk:    :class:`~PublicKey` object
        :param array: list of initialized mpz_t objects or serialized
                      EncryptedArray
        """
        self._pk = pk

        if array is None:
            self._array = [make_c_mpz_t() for bit in range(n)]
        else:
            if isinstance(array, str):
                stringified_array = json.loads(array)
                self._array = [deserialize_c_mpz_t(elem) \
                    for elem in stringified_array]
            else:
                self._array = array

        self._n  = n  # Size
        self._k  = 0  # Iterator

        self.__add__ = self.__xor__

    def __getitem__(self, i):
        """Getter

        :rtype: :class:`~EncryptedBit` object
        """
        c = self._array[i]
        b = make_c_mpz_t()
        assign_c_mpz_t(b, c)
        return EncryptedBit(self._pk, b)

    def __setitem__(self, i, value):
        """Setter

        :param value: mpz_t or :class:`~EncryptedBit` object
        """
        if isinstance(value, EncryptedBit):
            b = value._as_parameter_
            clear_c_mpz_t(self._array[i])
            assign_c_mpz_t(self._array[i], b)
        else:
            self._array[i] = value

    def __next__(self):
        """Iterator"""
        if self._k >= self._n:
            raise StopIteration()
        else:
            current = self.__getitem__[self._k]
            self._k += 1
            return current

    def next(self):
        """Iterator for Python 2"""
        return self.__next__()

    def recrypt(self, sk):
        """Recrypt ciphertext"""
        for c in self._array:
            lib_scarab.fhe_recrypt(c, self._pk, sk)

    def __len__(self):
        """Array size"""
        return self._n

    def __xor__(self, other_array):
        """Homomorphic bitwise XOR"""
        raw_results = []
        for a, b in zip(self._array, other_array):
            c = make_c_mpz_t()
            lib_scarab.fhe_add(c, a, b, self._pk)
            raw_results.append(c)
        result = EncryptedArray(len(raw_results), self._pk, array=raw_results)
        result._array = raw_results
        return result

    def __and__(self, other_array):
        """Homomorphic bitwise AND"""
        raw_results = []
        for a, b in zip(self._array, other_array):
            c = make_c_mpz_t()
            lib_scarab.fhe_mul(c, a, b, self._pk)
            raw_results.append(c)
        result = EncryptedArray(len(raw_results), self._pk, array=raw_results)
        result._array = raw_results
        return result

    def __add__(self, other):
        """XOR alias"""
        return self.__xor__(other)

    def __del__(self):
        """Clear array of mpz_t"""
        for c in self._array:
            clear_c_mpz_t(c)

    def __str__(self):
        """Serialize array to string"""
        serialized_array = [serialize_c_mpz_t(mpz) for mpz in self._array]
        return json.dumps(serialized_array)


class PublicKey(object):

    """Public Key"""

    def __init__(self, pk):
        """Create PublicKey object from raw public key

        Should be constructed with :func:`~generate_pair`

        :param pk: initialized fhe_pk_t object or a serialized string
        """
        if isinstance(pk, str):
            self._as_parameter_ = deserialize_c_fhe_pk_t(pk)
        else:
            self._as_parameter_ = pk

    def encrypt(self, plain, sk=None):
        """Encrypt message bit-by-bit

        :param plain : plaintext bit array or a single bit
        :type plain  : list of integers or integer
        :param sk    : secret key, if not None, uses recrypt
        :type sk     : :class:`~PrivateKey` object or None
        :rtype       : :class:`~EncryptedArray` or :class:`~EncryptedBit`
                       object
        """
        if hasattr(plain, '__len__'):

            # Prepare encrypted bits. The encrypt function
            # is deterministic, so we can reuse them if
            # we are not using recryption
            encrypted_zero = make_c_mpz_t()
            encrypted_one = make_c_mpz_t()
            lib_scarab.fhe_encrypt(encrypted_zero, self, 0)
            lib_scarab.fhe_encrypt(encrypted_one, self, 1)

            encrypted_array = EncryptedArray(len(plain), self)

            for i, bit in enumerate(plain):

                c = make_c_mpz_t()

                # If sk is None, then just assign the prepared
                # encrypted bit. Otherwise, recrypt before assigning.
                if int(bit) == 0:
                    if sk is not None:
                        lib_scarab.fhe_recrypt(encrypted_zero, self, sk)
                    assign_c_mpz_t(c, encrypted_zero)
                elif int(bit) == 1:
                    if sk is not None:
                        lib_scarab.fhe_recrypt(encrypted_one, self, sk)
                    assign_c_mpz_t(c, encrypted_one)
                else:
                    raise ValueError('Plaintext can only be 0 or 1.')

                encrypted_array[i] = c
            return encrypted_array
        else:
            c = make_c_mpz_t()
            lib_scarab.fhe_encrypt(c, self, int(plain))
            if sk is not None:
                lib_scarab.fhe_recrypt(c, self, sk)
            return EncryptedBit(self, c)

    def __str__(self):
        """Serialize public key to JSON string"""
        return serialize_c_fhe_pk_t(self._as_parameter_)

    def __del__(self):
        """Clear key"""
        clear_c_fhe_pk_t(self)


class PrivateKey(object):

    """Private Key"""

    def __init__(self, sk):
        """Create PrivateKey object from raw private key

        Should be constructed with :func:`~generate_pair`

        :param sk: initialized fhe_sk_t object
        """
        self._as_parameter_ = sk

    def decrypt(self, encrypted):
        """Decrypt the encrypted array

        :param encrypted: :class:`~EncryptedArray` or :class:`~EncryptedBit`
                          object
        :returns        : decrypted ciphertext
        :rtype          : list of integers or integer
        """
        if hasattr(encrypted, '__len__') and len(encrypted) > 1:
            bits = [c_int() for enc_bit in encrypted]
            for i, enc_bit in enumerate(encrypted):
                bits[i] = int(lib_scarab.fhe_decrypt(enc_bit, self))
            return bits

        else:
            return int(lib_scarab.fhe_decrypt(encrypted, self))

    def __del__(self):
        """Clear key"""
        clear_c_fhe_sk_t(self)


def generate_pair():
    """Generate public and private keypair

    >>> pk, sk = generate_pair()
    >>> sk.decrypt(pk.encrypt(1))
    1
    >>> sk.decrypt(pk.encrypt(0))
    0
    """
    pk, sk = make_c_fhe_pk_t(), make_c_fhe_sk_t()
    lib_scarab.fhe_keygen(pk, sk)
    return PublicKey(pk), PrivateKey(sk)



"""`libscarab` parameters taken directly from `parameters.h`

These need to be changed appropriately if corresponding values in
`parameters.h` are changed
"""

N      = 8
MU     = 4
NU     = 2**256
LOG_NU = 384
S1     = 8
S2     = 5
T      = 5            # Ceiling[Log[2, S2]] + 2
S      = 3            # Floor[Log[2, s2]] + 1





"""Library instance"""

import os

from ctypes import cdll
from ctypes.util import find_library


class Library(object):

    """Library loader"""

    libs = {}

    lib_paths = {
        'scarab': './libscarab.so',
        'gmp': './libgmp.so'
    }

    @staticmethod
    def load(name):
        """Load library if not loaded before"""
        if name not in Library.libs:
            lib_path = find_library(name)
            if lib_path is None:
                file_path = os.path.abspath(__file__)
                project_path = os.path.dirname(os.path.dirname(file_path))
                try:
                    default_lib_path = Library.lib_paths[name]
                    lib_path = os.path.join(project_path, default_lib_path)
                except:
                    pass
            Library.libs[name] = cdll.LoadLibrary(lib_path)

        return Library.libs[name]



"""Some low-level type definitions for `libscarab` and `GMP`"""

import json

from ctypes import POINTER, Structure, \
    c_ulong, c_ulonglong, c_int, c_char_p, string_at

from .loader import Library
from .predefs import *


lib_gmp = Library.load('gmp')
lib_scarab = Library.load('scarab')
libc = Library.load('libc')


class _c__mpz_struct(Structure):

    """mpz_struct (GMP Integer) definition"""

    _fields_ = [
        ('_mp_alloc',   c_int),
        ('_mp_size',    c_int),
        ('_mp_d',       POINTER(c_ulonglong))] # might be ulong / uint

# mpz_t definition
c_mpz_t = _c__mpz_struct * 1


def make_c_mpz_t():
    """Construct an mpz_t instance"""
    c = c_mpz_t()
    lib_gmp.__gmpz_init(c)
    return c


# mpz_t destructor
clear_c_mpz_t = lib_gmp.__gmpz_clear

# mpz_t assignment
assign_c_mpz_t = lib_gmp.__gmpz_set

def compare_c_mpz_t(a, b):
    """Compare mpz_t"""
    if isinstance(b, c_ulong):
        return lib_gmp.__gmpz_cmp_ui(a, b)
    elif isinstance(b, c_mpz_t):
        return lib_gmp.__gmpz_cmp(a, b)
    else:
        raise TypeError('Unknown `b` type')


base = 62


def serialize_c_mpz_t(mpz):
    """
    Serialize mpz_t

    :type mpz: c_mpz_t
    :rtype   : str
    """
    c_str = lib_gmp.__gmpz_get_str(None, base, mpz)
    result = string_at(c_str).decode('ascii')
    libc.free(c_str)
    return result

def deserialize_c_mpz_t(serialized_mpz):
    """
    Deserialize mpz_t

    :type serialized_mpz: str
    :rtype              : c_mpz_t
    """
    c_str = c_char_p(serialized_mpz.encode('ascii'))
    result = make_c_mpz_t()
    flag = lib_gmp.__gmpz_set_str(result, c_str, base)
    assert flag == 0
    return result


class _c__fhe_pk(Structure):

    """_fhe_pk (public key) definition"""

    _fields_ = [
        ('p',     c_mpz_t),
        ('alpha', c_mpz_t),
        ('c',     c_mpz_t * S1),
        ('B',     c_mpz_t * S1)]


# fhe_pk_t definition
c_fhe_pk_t = _c__fhe_pk * 1


def make_c_fhe_pk_t():
    """Construct an fhe_pk_t instance"""
    pk = c_fhe_pk_t()
    lib_scarab.fhe_pk_init(pk)
    return pk


# fhe_pk_t destructor
clear_c_fhe_pk_t = lib_scarab.fhe_pk_clear

def serialize_c_fhe_pk_t(pk):
    """
    Serialize c_fhe_pk_t

    :type pk: c_fhe_pk_t
    :returns: json dump of list of strings
    """
    result = {
        'p'    : serialize_c_mpz_t(pk[0].p),
        'alpha': serialize_c_mpz_t(pk[0].alpha),
        'c'    : [serialize_c_mpz_t(c_elem) for c_elem in pk[0].c],
        'B'    : [serialize_c_mpz_t(B_elem) for B_elem in pk[0].B]
    }
    return json.dumps(result)

def deserialize_c_fhe_pk_t(serialized_pk):
    """
    Deserialize fhe_pk_t

    :type serialized_pk: str
    :rtype             : fhe_pk_t
    """
    jsonified = json.loads(serialized_pk)
    result = make_c_fhe_pk_t()

    result[0].p = deserialize_c_mpz_t(jsonified['p'])
    result[0].alpha = deserialize_c_mpz_t(jsonified['alpha'])

    assert S1 == len(jsonified['c'])
    assert S1 == len(jsonified['B'])

    result[0].c = (c_mpz_t * S1)(*[deserialize_c_mpz_t(c_elem) for c_elem in jsonified['c']])
    result[0].B = (c_mpz_t * S1)(*[deserialize_c_mpz_t(B_elem) for B_elem in jsonified['B']])

    return result


class _c__fhe_sk(Structure):

    """_fhe_sk (private key) definition"""

    _fields_ = [
        ('p', c_mpz_t),
        ('B', c_mpz_t)]


# fhe_sk_t definition
c_fhe_sk_t = _c__fhe_sk * 1


def make_c_fhe_sk_t():
    """Construct an fhe_sk_t instance"""
    sk = c_fhe_sk_t()
    lib_scarab.fhe_sk_init(sk)
    return sk


# fhe_sk_t destructor
clear_c_fhe_sk_t = lib_scarab.fhe_sk_clear


"""Scarab wrapper unit tests"""

import json
import copy
import ctypes

from scarab import *

from scarab.loader import Library
from scarab.types import *

from nose.tools import *


lib_scarab = Library.load('scarab')


class TestTypes(object):

    """Test typedefs"""

    def test_sk_init(self):
        sk = c_fhe_sk_t()
        lib_scarab.fhe_sk_init(sk)
        lib_scarab.fhe_sk_clear(sk)

    def test_pk_init(self):
        pk = c_fhe_pk_t()
        lib_scarab.fhe_pk_init(pk)
        lib_scarab.fhe_pk_clear(pk)

    def test_mpz_compare(self):
        a = make_c_mpz_t()
        b = make_c_mpz_t()
        assert_true(compare_c_mpz_t(a, b) == 0)
        b = ctypes.c_ulong(50)
        assert_true(compare_c_mpz_t(a, b) != 0)

    def test_mpz_serialization(self):
        mpz = make_c_mpz_t()
        stringified = serialize_c_mpz_t(mpz)
        assert_equals(stringified, '0')

    def test_mpz_deserialization(self):
        pk, sk = generate_pair()
        mpz = pk.encrypt(1)
        stringified = serialize_c_mpz_t(mpz)
        mpz1 = deserialize_c_mpz_t(stringified)
        assert_true(compare_c_mpz_t(mpz, mpz1) == 0)

    def test_pk_serialization(self):
        pk, sk = generate_pair()
        stringified = serialize_c_fhe_pk_t(pk._as_parameter_)
        jsonified = json.loads(stringified)

    def test_pk_deserialization(self):
        array = [1, 1, 1, 0, 1, 0, 0, 1]
        pk, sk = generate_pair()
        stringified = serialize_c_fhe_pk_t(pk._as_parameter_)
        pk1 = deserialize_c_fhe_pk_t(stringified)
        encrypted_array = PublicKey(pk1).encrypt(array)
        decrypted_array = sk.decrypt(encrypted_array)
        assert_equals(decrypted_array, array)

class TestEncryptedBit(object):

    """EncryptedBit unit tests"""

    def setup(self):
        self.pk, self.sk = generate_pair()

    def test_recryption_ciphertext(self):
        """Test that the recrypted ciphertext is different
        when using recrypt function in EncryptedBit class."""
        for plain in [0, 1]:
            ciphertext = self.pk.encrypt(plain)
            c = make_c_mpz_t()
            assign_c_mpz_t(c, ciphertext._as_parameter_)
            ciphertext_copy = EncryptedBit(self.pk, c)
            ciphertext.recrypt(self.sk)
            assert_true(compare_c_mpz_t(ciphertext._as_parameter_,
                                        ciphertext_copy._as_parameter_) != 0)

    def test_recryption_plaintext(self):
        """Test that the recrypted ciphertext decrypts to
        the same plaintext when using recrypt function in
        EncryptedBit class.
        """
        for plain in [0, 1]:
            ciphertext = self.pk.encrypt(plain)
            ciphertext.recrypt(self.sk)
            decrypted = self.sk.decrypt(ciphertext)
            assert_equals(plain, decrypted)

class TestEncryptedArray(object):

    """EncryptedArray unit tests"""

    def setup(self):
        self.pk, self.sk = generate_pair()
        self.array = EncryptedArray(16, self.pk)

    def test_iteration(self):
        counter = 0
        for c in self.array:
            counter += 1
            assert_not_equals(c, None)
        assert_equals(counter, 16)

    def test_get(self):
        self.array[5]
        assert_raises(Exception, self.array, 16)

    def test_get(self):
        m = make_c_mpz_t()
        self.array[5] = m
        assert_true(compare_c_mpz_t(self.array[5], m) == 0)

    def test_len(self):
        assert_equals(len(self.array), 16)

    def test_recryption_ciphertext(self):
        """Test that the recrypted ciphertext is different
        when using recrypt function in EncryptedArray class."""
        encrypted_array = self.pk.encrypt([0, 1, 0, 0, 1, 0, 0, 1])
        old_array = [make_c_mpz_t() for i in range(8)]
        for a, b in zip(old_array, encrypted_array._array):
            assign_c_mpz_t(a, b)
        encrypted_array.recrypt(self.sk)
        for a, b in zip(old_array, encrypted_array._array):
            assert_true(compare_c_mpz_t(a, b) != 0)

    def test_recryption_plaintext(self):
        """Test that the recrypted ciphertext decrypts to
        the same plaintext when using recrypt function in
        EncryptedArray class.
        """
        encrypted_array = self.pk.encrypt([0, 1, 0, 0, 1, 0, 0, 1])
        encrypted_array.recrypt(self.sk)
        decrypted_array = self.sk.decrypt(encrypted_array)
        assert_equals([0, 1, 0, 0, 1, 0, 0, 1], decrypted_array)


class TestEncryption(object):

    """Test key generation and encryption"""

    def setup(self):
        self.pk, self.sk = generate_pair()

    def test_generate_pair(self):
        assert_true(compare_c_mpz_t(self.pk, make_c_mpz_t()) != 0)
        assert_true(compare_c_mpz_t(self.sk, make_c_mpz_t()) != 0)

    def test_bit_encryption(self):
        """Check that decryption of an encrypted bit again leads
        to the plaintext.
        """
        for plain in [0, 1]:
            c = self.pk.encrypt(plain)
            assert_true(isinstance(c, EncryptedBit))
            p = self.sk.decrypt(c)
            assert_equals(p, plain)

    @nottest
    def test_bit_encryption_determinism(self):
        """Check that encryption of the same plaintext over and over
        again leads to the same ciphertext.

        FIXME: Test on EncryptedArray
        """
        for plain in [0, 1]:
            c = self.pk.encrypt(plain)
            for i in range(100):
                same = self.pk.encrypt(plain)
                assert_true(compare_c_mpz_t(
                    c._as_parameter_, same._as_parameter_) == 0)

    def test_bit_recryption(self):
        """Check that when passing the secret key to PublicKey.encrypt()
        the encryption is done using recrypt and thus the same as when
        first encrypt without passing the secret key and then calling
        recrypt function on the EncryptedBit object.
        """
        for plain in [0, 1]:
            without_recrypt = self.pk.encrypt(plain)
            without_recrypt.recrypt(self.sk)
            with_recrypt = self.pk.encrypt(plain, self.sk)
            assert_true(compare_c_mpz_t(without_recrypt._as_parameter_,
                with_recrypt._as_parameter_) == 0)

    def test_array_encryption(self):
        """Check that decryption of an encrypted array again leads
        to the plaintext.
        """
        m = [0, 0, 0, 0, 0, 0, 0, 0]
        c = self.pk.encrypt(m)
        p = self.sk.decrypt(c)
        assert_true(compare_c_mpz_t(c[0], make_c_mpz_t()) != 0)
        assert_equals(m, p)

        m = [1, 0, 1, 0, 1, 0, 1, 0]
        c = self.pk.encrypt(m)
        p = self.sk.decrypt(c)
        assert_true(compare_c_mpz_t(c[0], make_c_mpz_t()) != 0)
        assert_equals(m, p)

        m = [1, 1, 1, 1, 1, 1, 1, 1]
        c = self.pk.encrypt(m)
        p = self.sk.decrypt(c)
        assert_true(compare_c_mpz_t(c[0], make_c_mpz_t()) != 0)
        assert_equals(m, p)

    def test_array_recryption(self):
        """Check that when passing the secret key to PublicKey.encrypt()
        the encryption is done using recrypt and thus different from the
        one done without passing the secret key. As EncryptedArray.recrypt()
        recrypts each bit independently, this will be not the same.
        """
        m = [1, 0, 1, 0, 1, 0, 1, 0]
        without_recrypt = self.pk.encrypt(m)
        with_recrypt = self.pk.encrypt(m, self.sk)
        for a, b in zip(without_recrypt._array, with_recrypt._array):
            assert_true(compare_c_mpz_t(a, b) != 0)

class TestHomomorphicOperations(object):

    """Test homomorphic AND, XOR, +"""

    def setup(self):
        self.pk, self.sk = generate_pair()

    def test_array_xor(self):
        def check_result(a, b):
            ea = self.pk.encrypt(a)
            eb = self.pk.encrypt(b)
            r = self.sk.decrypt(ea ^ eb)
            c = list(map(lambda t: t[0] ^ t[1], zip(a, b)))
            print (a, b, c, r)
            assert_equals(r, c)
        pairs = [
            ([0, 0, 0, 0], [0, 0, 0, 0]),
            ([0, 0, 1, 1], [0, 0, 1, 1]),
            ([1, 1, 1, 1], [1, 1, 1, 1]),
        ]
        for a, b in pairs:
            check_result(a, b)

    def test_array_and(self):
        def check_result(a, b):
            ea = self.pk.encrypt(a)
            eb = self.pk.encrypt(b)
            r = self.sk.decrypt(ea & eb)
            c = list(map(lambda t: t[0] & t[1], zip(a, b)))
            print (a, b, c, r)
            assert_equals(r, c)
        pairs = [
            ([0, 0, 0, 0], [0, 0, 0, 0]),
            ([0, 0, 1, 1], [0, 0, 1, 1]),
            ([1, 1, 1, 1], [1, 1, 1, 1]),
        ]
        for a, b in pairs:
            check_result(a, b)

    def test_bit_xor(self):
        def check_result(a, b):
            ea = self.pk.encrypt(a)
            eb = self.pk.encrypt(b)
            r = self.sk.decrypt(ea ^ eb)
            c = a ^ b
            print (a, b, c, r)
            assert_equals(r, c)
        pairs = [
            (0, 0),
            (0, 1),
            (1, 0),
            (1, 1),
        ]
        for a, b in pairs:
            check_result(a, b)

    def test_bit_and(self):
        def check_result(a, b):
            ea = self.pk.encrypt(a)
            eb = self.pk.encrypt(b)
            r = self.sk.decrypt(ea & eb)
            c = a & b
            print (a, b, c, r)
            assert_equals(r, c)
        pairs = [
            (0, 0),
            (0, 1),
            (1, 0),
            (1, 1),
        ]
        for a, b in pairs:
            check_result(a, b)


class TestSerialization(object):

    def test_public_key_serialization(self):
        array = [1, 1, 1, 0, 1, 0, 0, 1]
        pk, sk = generate_pair()
        stringified = str(pk)
        pk1 = PublicKey(stringified)
        encrypted_array = pk1.encrypt(array)
        decrypted_array = sk.decrypt(encrypted_array)
        assert_equals(decrypted_array, array)

    def test_encrypted_bit_serialization(self):
        pk, sk = generate_pair()
        a = pk.encrypt(1)
        stringified = str(a)
        b = EncryptedBit(pk, stringified)
        one = sk.decrypt(b)
        assert_equals(one, 1)

    def test_encrypted_array_serialization(self):
        array = [1, 1, 1, 0, 1, 0, 0, 1]
        pk, sk = generate_pair()
        enc_array = pk.encrypt(array)
        stringified = str(enc_array)
        enc_array1 = EncryptedArray(len(array), pk, stringified)
        array1 = sk.decrypt(enc_array1)
        assert_equals(array, array1)





from setuptools import setup


setup (name='pyscarab',
       version='0.1',
       description='Python bindings for libscarab',
       author='Bogdan Kulynych, Benjamin Lipp, Davide Kirchner',
       author_email='hello@hidden-markov.com, mail@benjaminlipp.de, davide.kirchner@yahoo.it',
       url='https://github.com/blindstore/pyscarab',
       packages=['scarab', 'scarab.tests'],
       license='MIT',

       keywords='crypto',

       classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Security :: Cryptography',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
       ],

       test_suite='nose.collector',
       test_requires=[
           'nose'
       ])
